# coding: utf-8

"""
    Jungfraujoch

    API to control Jungfraujoch developed by the Paul Scherrer Institute (Switzerland). Jungfraujoch is a data acquisition and analysis system for pixel array detectors, primarly PSI JUNGFRAU. Jungfraujoch uses FPGA boards to acquire data at high data rates. 

    The version of the OpenAPI document: 1.0.0-rc.48
    Contact: filip.leonarski@psi.ch
    Generated by OpenAPI Generator (https://openapi-generator.tech)

    Do not edit the class manually.
"""  # noqa: E501


from __future__ import annotations
import pprint
import re  # noqa: F401
import json

from pydantic import BaseModel, ConfigDict, Field, StrictBool, StrictInt, field_validator
from typing import Any, ClassVar, Dict, List, Optional, Union
from typing_extensions import Annotated
from typing import Optional, Set
from typing_extensions import Self

class ImageFormatSettings(BaseModel):
    """
    ImageFormatSettings
    """ # noqa: E501
    summation: StrictBool = Field(description="Enable summation of images to a given image_time If disabled images are saved according to original detector speed, but image count is adjusted ")
    geometry_transform: StrictBool = Field(description="Place module read-out into their location on composed detector and extend multipixels ")
    jungfrau_conversion: StrictBool = Field(description="Convert pixel value in ADU to photon counts/energy Only affects JUNGFRAU detector ")
    jungfrau_conversion_factor_ke_v: Optional[Union[Annotated[float, Field(le=500.0, strict=True, ge=0.001)], Annotated[int, Field(le=500, strict=True, ge=1)]]] = Field(default=None, description="Used to convert energy deposited into pixel to counts If not provided incident_energy_keV is used ", alias="jungfrau_conversion_factor_keV")
    bit_depth_image: Optional[StrictInt] = Field(default=None, description="Bit depth of resulting image (it doesn't affect the detector read-out value) If not provided value is adjusted automatically ")
    signed_output: Optional[StrictBool] = Field(default=None, description="Controls if pixels have signed output If not provided value is adjusted automatically ")
    mask_module_edges: StrictBool = Field(description="Mask 1 pixel on the module boundary ")
    mask_chip_edges: StrictBool = Field(description="Mask multipixels on chip boundary ")
    jungfrau_mask_pixels_without_g0: Optional[StrictBool] = Field(default=True, description="JUNGFRAU: mask pixels that don't operate in G0, but do operate in G1 and G1. This should be turned off for cases, where detector is operated at room temperature with long exposure time. ")
    apply_mask: StrictBool = Field(description="Masked pixels are set to special value in the images produced by Jungfraujoch ")
    jungfrau_pedestal_g0_rms_limit: Optional[Annotated[int, Field(strict=True, ge=0)]] = Field(default=100, description="Pixels with pedestal G0 RMS above the threshold are marked as masked pixels")
    __properties: ClassVar[List[str]] = ["summation", "geometry_transform", "jungfrau_conversion", "jungfrau_conversion_factor_keV", "bit_depth_image", "signed_output", "mask_module_edges", "mask_chip_edges", "jungfrau_mask_pixels_without_g0", "apply_mask", "jungfrau_pedestal_g0_rms_limit"]

    @field_validator('bit_depth_image')
    def bit_depth_image_validate_enum(cls, value):
        """Validates the enum"""
        if value is None:
            return value

        if value not in set([8, 16, 32]):
            raise ValueError("must be one of enum values (8, 16, 32)")
        return value

    model_config = ConfigDict(
        populate_by_name=True,
        validate_assignment=True,
        protected_namespaces=(),
    )


    def to_str(self) -> str:
        """Returns the string representation of the model using alias"""
        return pprint.pformat(self.model_dump(by_alias=True))

    def to_json(self) -> str:
        """Returns the JSON representation of the model using alias"""
        # TODO: pydantic v2: use .model_dump_json(by_alias=True, exclude_unset=True) instead
        return json.dumps(self.to_dict())

    @classmethod
    def from_json(cls, json_str: str) -> Optional[Self]:
        """Create an instance of ImageFormatSettings from a JSON string"""
        return cls.from_dict(json.loads(json_str))

    def to_dict(self) -> Dict[str, Any]:
        """Return the dictionary representation of the model using alias.

        This has the following differences from calling pydantic's
        `self.model_dump(by_alias=True)`:

        * `None` is only added to the output dict for nullable fields that
          were set at model initialization. Other fields with value `None`
          are ignored.
        """
        excluded_fields: Set[str] = set([
        ])

        _dict = self.model_dump(
            by_alias=True,
            exclude=excluded_fields,
            exclude_none=True,
        )
        return _dict

    @classmethod
    def from_dict(cls, obj: Optional[Dict[str, Any]]) -> Optional[Self]:
        """Create an instance of ImageFormatSettings from a dict"""
        if obj is None:
            return None

        if not isinstance(obj, dict):
            return cls.model_validate(obj)

        _obj = cls.model_validate({
            "summation": obj.get("summation"),
            "geometry_transform": obj.get("geometry_transform"),
            "jungfrau_conversion": obj.get("jungfrau_conversion"),
            "jungfrau_conversion_factor_keV": obj.get("jungfrau_conversion_factor_keV"),
            "bit_depth_image": obj.get("bit_depth_image"),
            "signed_output": obj.get("signed_output"),
            "mask_module_edges": obj.get("mask_module_edges") if obj.get("mask_module_edges") is not None else True,
            "mask_chip_edges": obj.get("mask_chip_edges") if obj.get("mask_chip_edges") is not None else True,
            "jungfrau_mask_pixels_without_g0": obj.get("jungfrau_mask_pixels_without_g0") if obj.get("jungfrau_mask_pixels_without_g0") is not None else True,
            "apply_mask": obj.get("apply_mask") if obj.get("apply_mask") is not None else False,
            "jungfrau_pedestal_g0_rms_limit": obj.get("jungfrau_pedestal_g0_rms_limit") if obj.get("jungfrau_pedestal_g0_rms_limit") is not None else 100
        })
        return _obj


